//console.cpp

#include <cssysdef.h>
#include "console.h"
#include "csutil/csstring.h"
#include <iutil/eventq.h>
#include <iutil/event.h>
#include <csutil/eventnames.h>

void cimConsole::ExecCallback::Execute(const char *cmd)
{
	csString cimCommand(cmd);
	if (cimCommand =="exit")
	{
	csRef<iEventQueue> q = csQueryRegistry<iEventQueue>(oreg);
	 if (q.IsValid()) q->GetEventOutlet()->Broadcast(csevQuit(oreg));
	}
}

bool cimConsole::Initialize(iObjectRegistry* obj_reg)
{
	object_reg = obj_reg;

	conout = csQueryRegistryOrLoad<iConsoleOutput> (object_reg,
  	"crystalspace.console.output.standard");
  if (!conout)
  {
    csReport (object_reg,
	    	CS_REPORTER_SEVERITY_ERROR, "cim.console",
		"Can't load the output console!");
    return false;
  }
  conin = csQueryRegistryOrLoad<iConsoleInput> (object_reg,
  	"crystalspace.console.input.standard");
  if (!conin)
  {
    csReport (object_reg,
	    	CS_REPORTER_SEVERITY_ERROR, "cim.console",
		"Can't load the input console!");
    return false;
  }
 

  conin->Bind (conout);
  conin->SetPrompt ("cim:");
  ExecCallback* cb = new ExecCallback ();
  cb->conout = conout;
  cb->oreg = object_reg;
  conin->SetExecuteCallback (cb);


  cb->DecRef ();
  conout->SetVisible (false);

  return true;
}



bool cimConsole::HandleEvent(iEvent &ev)
{

	return conin->HandleEvent(ev);

}

void cimConsole::Hide()
{
	visible= false;
	conout->SetVisible(false);
}

void cimConsole::Show()
{
	visible= true;
	conout->SetVisible(true);
}

void cimConsole::Toggle()
{
	visible=!visible;
	conout->SetVisible(visible);

}

bool cimConsole::IsVisible()
{
	return visible;

}